@@ -112,6 +112,7 @@ num_recent_games = integer(10, 100)
 last_gameid = integer
 game_holded = integer
 wm_maximized = boolean
+wm_fullscreen = boolean
 splashscreen = boolean
 date_format = string
 mouse_type = string
@@ -122,6 +123,7 @@ use_cardset_bottoms = boolean
 dragcursor = boolean
 save_games_geometry = boolean
 game_geometry = int_list(min=2, max=2)
+topmost_dialogs = boolean
 sound = boolean
 sound_mode = integer(0, 1)
 sound_sample_volume = integer(0, 128)
@@ -142,6 +144,7 @@ solver_iterations_output_step = integer
 solver_preset = string
 display_win_message = boolean
 language = string
+table_zoom = list
 
 [sound_samples]
 move = boolean
@@ -164,6 +167,7 @@ gamewon = boolean
 droppair = boolean
 redo = boolean
 dealwaste = boolean
+edge = boolean
 extra = boolean
 
 [fonts]
@@ -185,6 +189,7 @@ cards_2 = string
 samerank_1 = string
 samerank_2 = string
 not_matching = string
+keyboard_sel = string
 
 [timeouts]
 highlight_samerank = float(0.2, 9.9)
@@ -220,6 +225,7 @@ scale_cards = boolean
 scale_x = float
 scale_y = float
 auto_scale = boolean
+preview_scale = boolean
 spread_stacks = boolean
 preserve_aspect_ratio = boolean
 resampling = integer(0, 10)
@@ -287,6 +293,7 @@ class Options:
         ('last_gameid', 'int'),
         ('game_holded', 'int'),
         ('wm_maximized', 'bool'),
+        ('wm_fullscreen', 'bool'),
         ('splashscreen', 'bool'),
         ('date_format', 'str'),
         ('mouse_type', 'str'),
@@ -297,6 +304,7 @@ class Options:
         # ('save_cardsets', 'bool'),
         ('dragcursor', 'bool'),
         ('save_games_geometry', 'bool'),
+        ('topmost_dialogs', 'bool'),
         ('sound', 'bool'),
         ('sound_mode', 'int'),
         ('sound_sample_volume', 'int'),
@@ -320,6 +328,9 @@ class Options:
         # ('favorite_gameid', 'list'),
         ('display_win_message', 'bool'),
         ('language', 'str'),
+        # ('table_zoom', 'list'),
+        ('fontscale', 'str'),
+        # ('fontsizefactor', 'float'),
         ]
 
     def __init__(self):
@@ -415,6 +426,9 @@ class Options:
         self.translate_game_names = True
         self.display_win_message = True
         self.language = ''
+        self.table_zoom = [1.0, 0.0, 0.0]
+        self.fontscale = 'default'        # (kivy,  platform defaults)
+        # self.fontsizefactor = 1.0
         # sound
         self.sound = True
         self.sound_mode = 1
@@ -432,6 +446,7 @@ class Options:
             'dealwaste': True,
             'droppair': True,
             'drop': True,
+            'edge': True,
             'extra': True,
             'flip': True,
             'move': True,
@@ -469,6 +484,7 @@ class Options:
             'samerank_2':   '#0000ff',
             'hintarrow':    '#303030',
             'not_matching': '#ff0000',
+            'keyboard_sel': '#bf40bf',
             }
         # delays
         self.timeouts = {
@@ -490,7 +506,9 @@ class Options:
         self.last_gameid = 0            # last game played
         self.game_holded = 0            # gameid or 0
         self.wm_maximized = 1
+        self.wm_fullscreen = 0
         self.save_games_geometry = False
+        self.topmost_dialogs = True
         # saved games geometry (gameid: (width, height))
         self.games_geometry = {}
         self.game_geometry = (0, 0)  # game geometry before exit
@@ -505,6 +523,7 @@ class Options:
         self.scale_x = 1.0
         self.scale_y = 1.0
         self.auto_scale = True
+        self.preview_scale = True
         self.spread_stacks = False
         self.center_layout = True
         self.preserve_aspect_ratio = True
@@ -642,7 +661,6 @@ class Options:
         if isinstance(self.mahjongg_create_solvable, bool):
             # changed in v.1.1
             self.mahjongg_create_solvable = 2
-        pass
 
     def copy(self):
         opt = Options()
@@ -665,12 +683,13 @@ class Options:
 
         config['general']['recent_gameid'] = self.recent_gameid
         config['general']['favorite_gameid'] = self.favorite_gameid
-        visible_buttons = [b for b in self.toolbar_vars
-                           if self.toolbar_vars[b]]
-        config['general']['visible_buttons'] = visible_buttons
-        visible_status = [b for b in self.statusbar_vars
-                          if self.statusbar_vars[b]]
-        config['general']['visible_status'] = visible_status
+        config['general']['table_zoom'] = self.table_zoom
+        config['general']['visible_buttons'] = [
+            b for b, is_visible in self.toolbar_vars.items() if is_visible
+        ]
+        config['general']['visible_status'] = [
+            s for s, is_visible in self.statusbar_vars.items() if is_visible
+        ]
         if 'none' in config['general']['solver_presets']:
             config['general']['solver_presets'].remove('none')
 
@@ -699,7 +718,7 @@ class Options:
                 else:
                     config['cardsets'][str(key)] = val2
         for key in ('scale_cards', 'scale_x', 'scale_y',
-                    'auto_scale', 'spread_stacks',
+                    'auto_scale', 'preview_scale', 'spread_stacks',
                     'preserve_aspect_ratio', 'resampling'):
             config['cardsets'][key] = getattr(self, key)
 
@@ -784,8 +803,6 @@ class Options:
                     continue
                 for key, value in data.items():
                     if value is False:
-                        print_err('config file: validation error: '
-                                  'section: "%s", key: "%s"' % (section, key))
                         config[section][key] = None
 
         # general
@@ -812,6 +829,13 @@ class Options:
             except Exception:
                 traceback.print_exc()
 
+        table_zoom = self._getOption('general', 'table_zoom', 'list')
+        if table_zoom is not None:
+            try:
+                self.table_zoom = [float(i) for i in table_zoom]
+            except Exception:
+                traceback.print_exc()
+
         visible_buttons = self._getOption('general', 'visible_buttons', 'list')
         if visible_buttons is not None:
             for key in TOOLBAR_BUTTONS:
@@ -863,8 +887,8 @@ class Options:
                 self.timeouts[key] = val
 
         # cardsets
-        for key in self.cardset:
-            for key2 in self.cardset[key]:
+        for key, cur_cardset in self.cardset.items():
+            for key2 in cur_cardset:
                 if key2 > 0:
                     val = self._getOption('cardsets',
                                           str(key) + "_" + str(key2), 'list')
@@ -879,6 +903,7 @@ class Options:
                        ('scale_x', 'float'),
                        ('scale_y', 'float'),
                        ('auto_scale', 'bool'),
+                       ('preview_scale', 'bool'),
                        ('spread_stacks', 'bool'),
                        ('preserve_aspect_ratio', 'bool'),
                        ('resampling', 'int')):
